//;-------------------------------------------------------------------------
//; Copyright (c) 2001 Steven Freyder, David Helland, and Bruce D. Lightner.
//; Use subject to licensing restrictions described in "PICOWEB SERVER
//; SOFTWARE LICENSE AGREEMENT DATED JULY 14, 1999" (See file LICENSE.tt).
//;-------------------------------------------------------------------------

#include <windows.h>
#include <stdio.h>
#include <process.h>
#include "giveio.h"

BOOL SetPrivilege(
		  pAdjustTokenPrivileges pAdj,
		  HANDLE hToken,          		/* token handle */
		  LPCTSTR Privilege,      		/* Privilege to enable/disable */
		  BOOL bEnablePrivilege   		/* TRUE to enable.  FALSE to disable */
		 ) ;

__cdecl AccessIoPorts(void)
{
    HANDLE hProcess ;
    HANDLE hToken ;
    HINSTANCE m_hNTDLL ;
    HINSTANCE m_hADVDLL ;
    DWORD Status = 0 ;
    pZwSetInformationProcess pSetInfo ;
    pAdjustTokenPrivileges pAdj ;
    pOpenProcessToken pOpen ;

    m_hNTDLL= LoadLibrary("ntdll.dll") ;
    if (m_hNTDLL < (HINSTANCE) HINSTANCE_ERROR)  {
	goto completed ;
    }		

    pSetInfo = (pZwSetInformationProcess) GetProcAddress(m_hNTDLL, "ZwSetInformationProcess") ;
    if (!pSetInfo)  {
	goto completed ;
    }

    m_hADVDLL = LoadLibrary("advapi32.dll") ;
    if (m_hADVDLL < (HINSTANCE) HINSTANCE_ERROR)  {
	goto completed ;
    }		

    pOpen = (pOpenProcessToken) GetProcAddress(m_hADVDLL, "OpenProcessToken") ;
    if (!pOpen)  {
	goto completed ;
    }

    pAdj = (pAdjustTokenPrivileges) GetProcAddress(m_hADVDLL, "AdjustTokenPrivileges") ;
    if (!pAdj)  {
	goto completed ;
    }

    hProcess = OpenProcess(				/* obtain a handle to the current process */
			   PROCESS_ALL_ACCESS,		/* access */
			   FALSE,			/* handle inheritance option */
			   _getpid()	 		/* process identifier */
			  ) ;
    if (hProcess == NULL)  {
	Status = GetLastError() ;
	fprintf(stderr, "Error obtaining process handle: %d, %08X\n", Status, Status) ;
    }

    if (!OpenProcessToken(
			  GetCurrentProcess(),
			  TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY,
			  &hToken
			 ))  {
    }

    /* enable SetDebugPrivilege */
    if (!SetPrivilege(pAdj, hToken, SE_TCB_NAME, TRUE))  {
	fprintf(stderr, "Error setting privileges!\n") ;
	fprintf(stderr, "Hint: Needs privilege \"Act as part of the operating system\"\n") ;
        CloseHandle(hToken);
	return(1) ;
    }

    Status = pSetInfo(					/* modify the process */
		      hProcess, 			/* handle to process */
		      ProcessUserModeIOPL,		/* IOPL getting modified */
		      0,		     		/* clearing...I think */
		      0					/* size of buffer */
		     ) ;
    if (Status)  {
	fprintf(stderr, "Error setting process information: %d, %08X\n", Status, Status) ;
    }
    
    CloseHandle(hProcess);
    CloseHandle(hToken);
completed: ;
    return(Status);
}

BOOL SetPrivilege(
		  pAdjustTokenPrivileges pAdj,		/* pointer to adjust routine */
		  HANDLE hToken,          		/* token handle */
		  LPCTSTR Privilege,      		/* Privilege to enable/disable */
		  BOOL bEnablePrivilege   		/* TRUE to enable.  FALSE to disable */
		 )
{
    TOKEN_PRIVILEGES tp ;
    LUID luid ;
    TOKEN_PRIVILEGES tpPrevious ; 
    DWORD cbPrevious=sizeof(TOKEN_PRIVILEGES) ;

    if(!LookupPrivilegeValue(NULL, Privilege, &luid))  {
	return(FALSE) ;
    }

    /* first pass.  get current privilege setting */
    tp.PrivilegeCount = 1 ;
    tp.Privileges[0].Luid = luid ;
    tp.Privileges[0].Attributes = 0 ;

    pAdj(
	 hToken,
	 FALSE,
	 &tp,
	 sizeof(TOKEN_PRIVILEGES),
	 &tpPrevious,
	 &cbPrevious
	) ;

    if (GetLastError() != ERROR_SUCCESS)  {
	return(FALSE) ;
    }

    /* second pass.  set privilege based on previous setting */
    tpPrevious.PrivilegeCount = 1 ;
    tpPrevious.Privileges[0].Luid = luid ;

    if(bEnablePrivilege)  {
        tpPrevious.Privileges[0].Attributes |= (SE_PRIVILEGE_ENABLED) ;
    }
    else {
        tpPrevious.Privileges[0].Attributes ^=
	    (SE_PRIVILEGE_ENABLED & tpPrevious.Privileges[0].Attributes) ;
    }

    pAdj(
	 hToken,
	 FALSE,
	 &tpPrevious,
	 cbPrevious,
	 NULL,
	 NULL
	) ;
    if (GetLastError() != ERROR_SUCCESS)  {
	return(FALSE) ;
    }
    return(TRUE) ;
}
